/*
 * WSPR_Beacon_3.c
 *
 * Created: 25.01.2021 11:32:26
 * Author: R0AEK
 * Ver - 1.05
 */ 

//   EEPROM
/*
00...11		 					1-off; 2-on;
12...17									6-;
18...21										4-;
22-23		  			2-;
24			  			0-DS3231; 1-GPS;
25			   	0-none; 1-skip_1_period;2-skip_2_period;3-skip_3_period;4-skip_4_period
26			  BPF				0-off; 1-on;
27...30		  Xtal			4-;
31			  PLLA					0-600 MHz; 1-900 MHz;
32			    		0-250; 1-500; 2-750; 3-1000; 4-1250; 5-1500; 6-1750; 7-2000; 8-2250;
33			  			0-; 1-i2c 1602_T LED on; 2-i2c 1602_T LED off; 3-i2c 1602_AT LED on; 4-i2c 1602_AT LED off; 5-i2c OLED_12864/32;
34			  Xtal			0-25 MHz; 1-27 MHz;
40...201	WSPR 
300...315	ABCD    extBPF
316			   PB1  	1/0
*/

//     UART
/*
A1 -     
2 -     "Active bands"  EEPROM
A3 -     "Active bands"  EEPROM

B1 -     "TX options"  EEPROM
B2 -     "TX options"  EEPROM
B3 -      "WSPR data"  EEPROM
B4 -      "WSPR data"  EEPROM
B5 -      "WSPR data"  EEPROM
B6 -         
B7 -         
B8 -        PB1  

C1 -     "Clock generator Si5351"  EEPROM
C2 -     "Clock generator Si5351"  EEPROM

D1 -    EEPROM     

E1 -       Ds3231

F1 -      

G1 -         
G2 -        
G3 -     *

H1 -      Si5351  - 500 
H2 -      Si5351  - 500 
H3 -      Si5351  - 100 
H4 -      Si5351  - 100 
H5 -      Si5351  - 5 
H6 -      Si5351  - 5 

I0 -        GPS    
I1 -        GPS    
I3 -    GPS

J1 -    ABCD code     
J2 -    ABCD code     
*/

#include "Main.h"

/*   ---------------------------------------*/

	const char logo1[] PROGMEM				= {"WSPR Beacon-3   "};	//   LCD 
	const char logo2[] PROGMEM				= {"Based on Si5351 "};	//-
	const char logo3[] PROGMEM				= {"By R0AEK        "};	//-
	const char verInfo[] PROGMEM			= {"Version-1.05(h) "};	// ,    OLED   

	const uint16_t	WSPR_Ch_TX_Delay[6]		= {	//      WSPR  !!!    !!!  -  !!!
												// 12000/8192 = 0.682666 ,       = 110.591892 
												678,	// +     (  "L") + RTC,  678
												576,	// +     (  "L") + GPS,  576
												637,	// +  1602 i2c + RTC,  637
												536,	// +  1602 i2c + GPS,  536
												658,	// +  OLED ssd1306 i2c + RTC,  658
												558};	// -  OLED ssd1306 i2c + GPS,  558
	const uint32_t BandsFreq[12]			= {
												1836600,	//160m
												3568600,	// 80m
												5287200,	// 60m
												7038600,	// 40m
												10138700,	// 30m
												14095600,	// 20m
												18104600,	// 17m
												21094600,	// 15m
												24924600,	// 12m
												28124600,	// 10m
												50293000,	//  6m
												144489000};	//  2m
	
	const uint32_t freq4FSK					= 1.4648;	//  4FSK  -    WSPR = 1.4648 
	const uint32_t StartFreq				= 1000000;	//   si5351 (1 )

	float XTAL_CORR							= 0;		//    

	char abcdCode[12]						= "";		// ABCD code
	uint32_t TXfreq							= 0;		// WSPR   si5351
	uint32_t freqOffset						= 1500;		//   (  1500 )
	uint8_t Bands[12];									//  
	uint8_t WSPR_Data[161];								// WSPR 
	static uint8_t dat_8_EEPROM				= 0xFF;		//    EEPROM

	uint8_t BandCount						= 1;		// 
	uint8_t Uses6mBand						= 0;		//   6m 0-no 1-yes
	uint8_t Uses2mBand						= 0;		//   2m 0-no 1-yes
	uint8_t BandPrev						= 0;		//  
	uint8_t TimeModule						= 0;		//      0-DS3231 1-GPS_NEO-6x
	uint8_t DisplayModule					= 0;		//    1602 0-LED's 1-i2c led_on 2-i2c led_off
	uint8_t FirstTX							= 1;		//  ,     ,   
	uint8_t TransmitDelay					= 0;		//     
	uint8_t PrevTimeStartTX					= 0;		//     
	unsigned long LED_Status_Blink			= 0;		//   
	uint8_t UseExtBPF						= 0;		//  extBPF
	uint8_t PA_Pin_Log						= 1;		//      

	extern unsigned char i2c_1602;						//i2c     1602 (0x4E - PCF8574T, 0x7E - PCF8574AT)
	extern uint8_t flagConnectToPC;						//    
	extern uint32_t XTAL_FREQ;							//   
	extern uint32_t FREQ_PLLA;							// - ( 900 ,    !!!)
	extern uint8_t	GPSDataStringReceived;				//    GPS
	extern uint8_t OLED_ADR;							//  i2c  OLED 
	
/*END    ----------------------------------------*/

/* --------------------------------------------------------------------*/

char* utoa_builtin_div(int value, char *buffer)
{
	/*  ,     10*/
	buffer += 8;
	// 11      32-  
	//    
	*--buffer = 0;
	do
	{
		*--buffer = value % 10 + '0';
		value /= 10;
	}
	while (value != 0);
	return buffer;
}

void ABCD_Off(void)
{
	/*  ABCD  0*/
	PORTX_D &= ~(1 << ABCD_A); // PD5
	PORTX_D &= ~(1 << ABCD_B); // PD6
	PORTX_D &= ~(1 << ABCD_C); // PD7
	PORTX_B &= ~(1 << ABCD_D); // PB0
}

void Strobe_L_LED(void) 
{
	/*  "L"     WSPR */
	
	if (TimeModule == 0)
	{
		// 
		if (LED_Status_Blink >= 2500)
			LED_Status_Blink = 0;
		else
			LED_Status_Blink++;
		
		if (LED_Status_Blink == 0)
			//  
			PORTX_B |= (1 << LED_Status);
		
		if (LED_Status_Blink == 100)
			//  
			PORTX_B &= ~(1 << LED_Status);
		
		//if (LED_Status_Blink == 200)
			////  
			//PORTX_B |= (1 << LED_Status);
		//
		//if (LED_Status_Blink == 300)
			////  
			//PORTX_B &= ~(1 << LED_Status);
	}
	
	if(TimeModule == 1)
	{
		// 
		if (LED_Status_Blink >= 100000)
			LED_Status_Blink = 0;
		else
			LED_Status_Blink++;
		
		if (LED_Status_Blink == 0)
			//  
			PORTX_B |= (1 << LED_Status);
		
		if (LED_Status_Blink == 2000)
			//  
			PORTX_B &= ~(1 << LED_Status);
		
		if (LED_Status_Blink == 7000)
			//  
			PORTX_B |= (1 << LED_Status);
		
		if (LED_Status_Blink == 9000)
			//  
			PORTX_B &= ~(1 << LED_Status);
	}
}

void Transmit_WSPR (void)
{
	// 
	uint32_t fOut;
	char strTemp[10];
	
	//  PA
	if (PA_Pin_Log == '1')
		PORTX_B |= (1 << PA_Pin);
	if (PA_Pin_Log == '0')
		PORTX_B &= ~(1 << PA_Pin);
	
	if(Bands[BandCount-1] != '2')
	{
		while(Bands[BandCount-1] != '2')
		{
			BandCount++;
			if(BandCount >= 13)
				BandCount = 1;
		}
	}
	
	/*  extBPF    
	   PD5/PD6/PD7/PB0*/
	if (UseExtBPF == '1')
		ABCDDecode();
	
	if (DisplayModule != 0)
	{
		{
			//   LCD 
			if ((DisplayModule >= 1)&&(DisplayModule <= 4))
			{
				//   LCD 
				LCD_set_pos(0,0);
				switch (BandCount)
				{
					case 1:  LCD_send_str("Transmit:   160m");
					break;
					case 2:  LCD_send_str("Transmit:    80m");
					break;
					case 3:  LCD_send_str("Transmit:    60m");
					break;
					case 4:  LCD_send_str("Transmit:    40m");
					break;
					case 5:  LCD_send_str("Transmit:    30m");
					break;
					case 6:  LCD_send_str("Transmit:    20m");
					break;
					case 7:  LCD_send_str("Transmit:    17m");
					break;
					case 8:  LCD_send_str("Transmit:    15m");
					break;
					case 9:  LCD_send_str("Transmit:    12m");
					break;
					case 10: LCD_send_str("Transmit:    10m");
					break;
					case 11: LCD_send_str("Transmit:     6m");
					break;
					case 12: LCD_send_str("Transmit:     2m");
					break;
					default: ;
				}
				LCD_set_pos(0,1);
				LCD_send_str("Sym ch:");
			}
			
			//   OLED 
			if ((DisplayModule == 5)||(DisplayModule == 6))
			{
				//   OLED 
				// 
				switch (BandCount)
				{
					case 1:   PlotText(PSTR(" TX:  160m"), 0, 0);
					break;
					case 2:   PlotText(PSTR(" TX:   80m"), 0, 0);
					break;
					case 3:   PlotText(PSTR(" TX:   60m"), 0, 0);
					break;
					case 4:   PlotText(PSTR(" TX:   40m"), 0, 0);
					break;
					case 5:   PlotText(PSTR(" TX:   30m"), 0, 0);
					break;
					case 6:   PlotText(PSTR(" TX:   20m"), 0, 0);
					break;
					case 7:   PlotText(PSTR(" TX:   17m"), 0, 0);
					break;
					case 8:   PlotText(PSTR(" TX:   15m"), 0, 0);
					break;
					case 9:   PlotText(PSTR(" TX:   12m"), 0, 0);
					break;
					case 10:  PlotText(PSTR(" TX:   10m"), 0, 0);
					break;
					case 11:  PlotText(PSTR(" TX:    6m"), 0, 0);
					break;
					case 12:  PlotText(PSTR(" TX:    2m"), 0, 0);
					break;
					default: ;
				}
				
				for (int i=0;i<10;i++)
					strTemp[i] = ' ';
				int y = 0;
				for (int i=0;i<10;i++)
				{
					PlotChar(strTemp[i],2,y);
					y += 13;
				}
			}
		}
	}
	else
	{
		//  
		PORTX_B |= (1 << LED_Status);
	}
	
	//   
	TXfreq = BandsFreq[BandCount-1];
	
	//       10...190 
	dat_8_EEPROM = eeprom_read_byte((uint8_t*)32);
	if (dat_8_EEPROM == '9')
		freqOffset = 1400 + rand() % 191;
		
	/*
		WSPR 
		      ,     
	*/
	//for (int i = 0; i < 20; i++) // 
	for (int i = 0; i < 162; i++) //
	{
		/*
		 
		    4-FSK  :
		fOut() = fTX() + (n  * 1.4648())  n -   0..3
		*/
		fOut = TXfreq + (uint32_t)(WSPR_Data[i] * freq4FSK);
		si5351aSetFrequency(fOut + freqOffset);
		
		if (DisplayModule != 0)
		{
			if ((DisplayModule >= 1)&&(DisplayModule <= 4))
			{
				//   LCD 
				LCD_set_pos(7,1);
				LCD_send_char(WSPR_Data[i]); // 
				if(i+1 < 10)
				{
					LCD_send_str("   ");
				}
				if((i+1 >= 10)&&(i+1 <= 99))
				{
					LCD_send_str("  ");
				}
				if(i+1 > 99)
				{
					LCD_send_str(" ");
				}
				LCD_send_str(utoa_builtin_div(i+1,0));
				LCD_send_str("/162");
			}
			
 			if ((DisplayModule == 5)||(DisplayModule == 6))
 			{
				//   OLED 
				//    OLED 
				//uint8_t x = i/20.*100; // 
				uint8_t x = i/161.*100;	//
 				
				if (x <= 99)
				{
 					sprintf(strTemp, "    %i%%\n", x);
				}
 				if (x >= 100)
 					sprintf(strTemp, "   %i%%\n", x);
					 
	 			int y = 0;
	 			for (int i=0;strTemp[i]!='\n';i++)
	 			{
		 			PlotChar(strTemp[i],2,y);
		 			y += 13;
	 			}
 			}
		}
		
		/*    -----------------------------*/
		
		/*  + RTC*/
		if ((TimeModule == 0)&&(DisplayModule == 0))
			_delay_ms(WSPR_Ch_TX_Delay[0]);
		
		/*  + GPS*/
		if ((TimeModule == 1)&&(DisplayModule == 0))
			_delay_ms(WSPR_Ch_TX_Delay[1]);
		
		/* 1602 i2c + RTC*/
		if ((TimeModule == 0)&&((DisplayModule >= 1)&&(DisplayModule <= 4)))
			_delay_ms(WSPR_Ch_TX_Delay[2]);
		
		/* 1602 i2c + GPS*/
		if ((TimeModule == 1)&&((DisplayModule >= 1)&&(DisplayModule <= 4)))
			_delay_ms(WSPR_Ch_TX_Delay[3]);
		
		/* OLED i2c + RTC*/
		if((TimeModule == 0)&&((DisplayModule == 5)||(DisplayModule == 6)))
			_delay_ms(WSPR_Ch_TX_Delay[4]);
		
		/* OLED i2c + GPS*/
		if((TimeModule == 1)&&((DisplayModule == 5)||(DisplayModule == 6)))
			_delay_ms(WSPR_Ch_TX_Delay[5]);
		
		/*END     -------------------------*/
	}
	
	//   
	si5351aSetFrequency(StartFreq);
	
	if (DisplayModule == 0)
	{
		//  
		PORTX_B &= ~(1 << LED_Status);
	}
	
	if ((DisplayModule >= 1)&&(DisplayModule <= 4))
	{
		LCD_set_pos(0,0);
		LCD_send_str("Waiting 00:00:00");
		LCD_set_pos(0,1);
		switch (TimeModule)
		{
			case 0:  LCD_send_str("RTC DS-3231     ");
			break;
			case 1:  LCD_send_str("GPS   Sat:-- Q:-");
			break;
			default: ;
		}
	}
	
	if ((DisplayModule == 5)||(DisplayModule == 6))
	{
		OLED_clear();
		PlotText(PSTR("  Waiting  "), 0, 0);
	}
	
	//  PA
	if (PA_Pin_Log == '1')
		PORTX_B &= ~(1 << PA_Pin);
	if (PA_Pin_Log == '0')
		PORTX_B |= (1 << PA_Pin);
	
	//ABCD  0
	ABCD_Off();
	
	//  
	BandPrev = BandCount;
	//  
	BandCount++;
	if(BandCount >= 13)
		BandCount = 1;
}

void ReadXtallPLLAfrommEEPROM(void)
{
	/*  XTall  PLLA  EEPROM*/
	
	//     EEPROM
	XTAL_CORR = eeprom_read_float((float*)27);
	
	//  Xtal (   si5351)
	dat_8_EEPROM = eeprom_read_byte((uint8_t*)34);
	
	switch(dat_8_EEPROM)
	{
		case '0':  XTAL_FREQ = 25000000 - (uint32_t)XTAL_CORR;
		break;
		case '1':  XTAL_FREQ = 27000000 - (uint32_t)XTAL_CORR;
		break;
		default: ;
	}
	
	//  PLLA
	dat_8_EEPROM = eeprom_read_byte((uint8_t*)31);
	
	switch(dat_8_EEPROM)
	{
		case '0':  FREQ_PLLA = 600000000;
		break;
		case '1':  FREQ_PLLA = 900000000;
		break;
		default: ;
	}
	
	//  FreqOffset
	dat_8_EEPROM = eeprom_read_byte((uint8_t*)32);
	
	switch(dat_8_EEPROM)
	{
		case '0':  freqOffset = 1400;
		break;
		case '1':  freqOffset = 1425;
		break;
		case '2':  freqOffset = 1450;
		break;
		case '3':  freqOffset = 1475;
		break;
		case '4':  freqOffset = 1500;
		break;
		case '5':  freqOffset = 1525;
		break;
		case '6':  freqOffset = 1550;
		break;
		case '7':  freqOffset = 1575;
		break;
		case '8':  freqOffset = 1600;
		break;
		case '9':  freqOffset = 1400;	//           1400...1600 
		break;
		default: ;
	}
}

int main (void)
{
	//  
	{
		DDRX_B  = 0xFF;	//  
		PORTX_B = 0x00;	//  0
		
		DDRX_C  = 0xFE;	//0    2-7   
		PORTX_C = 0x00;	//  0
		
		DDRX_D  = 0xF8;	// 5/6/7  
		
		ABCD_Off();
	}
	
	//  !!!!!!!!!! 
	{
		i2cInit();
		USART_Init(MYUBRR);
	}
	
	///    
	{
		dat_8_EEPROM = eeprom_read_byte((uint8_t*)33);
		if ((dat_8_EEPROM != '0')&&(dat_8_EEPROM != '1')&&(dat_8_EEPROM != '2')&&(dat_8_EEPROM != '3')&&(dat_8_EEPROM != '4')&&(dat_8_EEPROM != '5'))
		{
			//   ,     ( L)
			eeprom_write_byte((uint8_t*)33,'0');
		}
	}
	
	//   
	{
		switch (dat_8_EEPROM)
		{
			case '0':	DisplayModule = 0;	//Status Led
			break;
			case '1':	DisplayModule = 1;	//1602 i2c T Led on
						i2c_1602 = 0x4E;
						LCD_ini();
			break;
			case '2':	DisplayModule = 2;	//1602 i2c T Led off
						i2c_1602 = 0x4E;
						LCD_ini();
			break;
			case '3':	DisplayModule = 3;	//1602 i2c AT Led on
						i2c_1602 = 0x7E;
						LCD_ini();
			break;
			case '4':	DisplayModule = 4;	//1602 i2c AT Led off
						i2c_1602 = 0x7E;
						LCD_ini();
			break;
			case '5':	DisplayModule = 5;	//OLED i2c
						OLED_ADR = 0x78;
						OLED_init();
			break;
			case '6':	DisplayModule = 6;	//OLED i2c
						OLED_ADR = 0x3C;
						OLED_init();
			break;
			
			default:	DisplayModule = 0;	//Status Led
		}
	}
	
	//     EEPROM
	{
		//  
		{
			for (int i=0;i<=11;i++)
			{
				Bands[i] = eeprom_read_byte((uint8_t*)i);
			}
		}
		
		//    
		{
			TransmitDelay = eeprom_read_byte((uint8_t*)25);
		}
		
		// WSPR 
		{
			for (int i = 40; i < 202; i++)
			{
				//      
				WSPR_Data[i-40] = eeprom_read_byte((uint8_t*)i);
			}
		}
		
		// ABCD code
		{
			UseExtBPF = eeprom_read_byte((uint8_t*)26);
			for (int i=300;i<310;i++)
			{
				abcdCode[i-300] = eeprom_read_byte((uint8_t*)i);
			}
		}
		
		//    PB1  
		{
			PA_Pin_Log = eeprom_read_byte((uint8_t*)316);
		}
		
		///  Xtal (   si5351)
		{
			dat_8_EEPROM = eeprom_read_byte((uint8_t*)26);
			if ((dat_8_EEPROM != '0')&&(dat_8_EEPROM != '1'))
			{
				//   ,     (25.000 MHz)
				eeprom_write_byte((uint8_t*)26,'0');
			}
		}
		
		///  PLLA
		{
			dat_8_EEPROM = eeprom_read_byte((uint8_t*)31);
			if ((dat_8_EEPROM != '0')&&(dat_8_EEPROM != '1'))
			{
				//   ,     (600.000 MHz)
				eeprom_write_byte((uint8_t*)31,'0');
			}
			
			ReadXtallPLLAfrommEEPROM();
		}
		
		//      
		{
			dat_8_EEPROM = eeprom_read_byte((uint8_t*)24);
			
			switch (dat_8_EEPROM)
			{
				case '0':  TimeModule = 0;					//DS-3231
				break;
				case '1':  {TimeModule = 1; SUART_Init();}	//GPS
				break;
				default: ;
			}
		}
	}
	
	//  PA
	{
		if (PA_Pin_Log == '1')
			PORTX_B &= ~(1 << PA_Pin);
		if (PA_Pin_Log == '0')
			PORTX_B |= (1 << PA_Pin);
	}
	
	/*,    6/2m,  ,    ,
	 ,    */
	{
		dat_8_EEPROM = eeprom_read_byte((uint8_t*)10);
		if (dat_8_EEPROM == '2')
			Uses6mBand = '1';
		else
			i2cSendRegister(SI_CLK1_CONTROL, 0x80);
		
		dat_8_EEPROM = eeprom_read_byte((uint8_t*)11);
		if (dat_8_EEPROM == '2')
			Uses2mBand = '1';
		else
			i2cSendRegister(SI_CLK2_CONTROL, 0x80);
	}
	
	// si5351   
	si5351aSetFrequency(StartFreq);
	
	//  UART        1602
	{
		switch (DisplayModule)
		{
			case 0:  SendStringDataToUSART("WSPR0\n\n");
			break;
			case 1:  SendStringDataToUSART("WSPR1\n\n");
			break;
			case 2:  SendStringDataToUSART("WSPR2\n\n");
			break;
			case 3:  SendStringDataToUSART("WSPR3\n\n");
			break;
			case 4:  SendStringDataToUSART("WSPR4\n\n");
			break;
			case 5:  SendStringDataToUSART("WSPR5\n\n");
			break;
			case 6:  SendStringDataToUSART("WSPR6\n\n");
			break;
			default: SendStringDataToUSART("WSPR0\n\n");;
		}
	}

	//    
	{
		//LCD 1602
		{
			if ((DisplayModule >= 1)&&(DisplayModule <= 4))
			{
				LCD_set_pos(0,0);
				LCD_send_str("Waiting 00:00:00");
				LCD_set_pos(0,1);
				switch (TimeModule)
				{
					case 0:  LCD_send_str("RTC DS-3231     ");
					break;
					case 1:  LCD_send_str("GPS   Sat:-- Q:-");
					break;
					default: ;
				}
			}
		}
		
		//OLED 128x64/128x32
		{
		}
	}
	
	//  
	sei();
	
	//  UART
	//UART_Monitor(logo1);
	//UART_Monitor(verInfo);
	
	/* -----------------------------------------------------------*/
	{
		while(1)
		{
			/*  /  UART*/
			handlerTXUSART();
			handlerRXUSART();
			
			// WSPR    -----------------------------------------------------------------------------------------*/
			{
				if (flagConnectToPC == 0)
				{
					//   RTC DS-3231
					if (TimeModule == 0)
					{
						//    RTC DS-3231
						if (DS3231_Read_Time() == 1)
						{
							if ((TransmitDelay != '0')&&(FirstTX == 0))
							{	//      
								switch (TransmitDelay)
								{
									case '1':  if (PrevTimeStartTX == 2) {PrevTimeStartTX = 0; Transmit_WSPR();};	//  
									break;
									case '2':  if (PrevTimeStartTX == 3) {PrevTimeStartTX = 0; Transmit_WSPR();};	//  
									break;
									case '3':  if (PrevTimeStartTX == 4) {PrevTimeStartTX = 0; Transmit_WSPR();};	//  
									break;
									case '4':  if (PrevTimeStartTX == 5) {PrevTimeStartTX = 0; Transmit_WSPR();};	//  
									break;
									default: ;
								}
							}
							else
							{
								if (FirstTX == 1)
								{
									//      
									FirstTX = 0;
								}
								
								// 
								Transmit_WSPR();
							}
							//_delay_ms(100);
						}
					}
					
					//   GPS-NEO-6x
					if (TimeModule == 1)
					{
						//  GPS   ,      GPS-NEO-6M
						if (GPSDataStringReceived == 1)
						{
							Parsing_GPS_Message();
						}
					}
					
					//  "L" (PB5)     WSPR 
					if (DisplayModule == 0)
					{
						Strobe_L_LED();
					}
				}
			}
			//END  WSPR    -------------------------------------------------------------------------------------*/
		}
	}
	/*END  -------------------------------------------------------*/
}

/* ---------------------------------------------------*/

ISR(USART_RX_vect)
{
	/*   USART_RX_vect*/
	bufRX [rxIn++] = UDR0; //    
	rxIn &= BUFMASKRX;
}

/*END  -----------------------------------------------*/

