/*
 * SoftUART.c
 *
 * Author(s)...: Pashgan http://ChipEnable.Ru
 * Description.: SoftUART driver for AVR.
 *
 * Created: 25.01.2021 11:32:26
 * Mod:		R0AEK
 * Ver		1.05
 *
 * Software UART - only RX
 *
 */

#include "SoftUART.h"

#define TRUE  1
#define FALSE 0

static volatile uint8_t 	flag_rx_ready;
static uint8_t				flag_rx_waiting_for_stop_bit;
static uint8_t 	            flag_rx_off;
static uint8_t 	            rx_mask;
static uint8_t 	            timer_rx_ctr;
static uint8_t 	            bits_left_in_rx;

#define get_rx_pin_status() PINX_C & (1<<Soft_UART_RX_PIN)
#define TIMER_TCNT_VALUE	(0xff - ((F_CPU/(9600L*3*8L)) - 1))

void SUART_Init(void)
{
	/* */
	flag_rx_ready				 = FALSE;
	flag_rx_waiting_for_stop_bit = FALSE;
	flag_rx_off					 = FALSE;
	
	/* */
	DDRX_C  &= ~(1<<Soft_UART_RX_PIN);	// RX  
	PORTX_C |= (1<<Soft_UART_RX_PIN);	//    RX
	
	/*  IMER2*/
	TCNT2  = TIMER_TCNT_VALUE;
	TCCR2A = (0<<COM2A1)|(0<<COM2A0)|(0<<COM2B1)|(0<<COM2B0)|(0<<WGM21)|(0<<WGM20);
	TCCR2B = (0<<FOC2A)|(0<<FOC2B)|(0<<CS22)|(1<<CS21)|(0<<CS20);
	TIMSK2 = (0<<OCIE2B)|(0<<OCIE2A)|(1<<TOIE2); //  2  
}

void SUART_TurnRxOn(void)
{
	flag_rx_off = FALSE;
	//TIMSK2 |= (1<<TOIE2);
}

void SUART_TurnRxOff(void)
{
	flag_rx_off = TRUE;
	//TIMSK2 |= (0<<TOIE2);
}

/*    TIMER2----------------------*/
ISR(TIMER2_OVF_vect)
{
	uint8_t start_bit; 
	uint8_t flag_in;
	/*uint8_t rx_data;*/
	static uint16_t  internal_rx_buffer;
	
	TCNT2 += TIMER_TCNT_VALUE;
	
	/* rx */
	if (flag_rx_off == FALSE)
	{
		if (flag_rx_waiting_for_stop_bit)
		{
			if (--timer_rx_ctr == 0)
			{
				flag_rx_waiting_for_stop_bit = FALSE;
				flag_rx_ready = FALSE;
				/*rx_data = (uint8_t)internal_rx_buffer;*/
				/*    */
				Receive_GPS_Data(/*rx_data*/(uint8_t)internal_rx_buffer);
			}
		}
		else
		{   // rx test busy
			if (flag_rx_ready == FALSE)
			{
				start_bit = get_rx_pin_status();
				if (start_bit == 0)
				{
					flag_rx_ready = TRUE;
					internal_rx_buffer = 0;
					timer_rx_ctr = 3 + 1;
					bits_left_in_rx = 8;
					rx_mask = 1;
				}
			}
			else
			{	// rx busy
				if (--timer_rx_ctr == 0)
				{
					timer_rx_ctr = 3;
					flag_in = get_rx_pin_status();
					if ( flag_in ){
						internal_rx_buffer |= rx_mask;
					}
					rx_mask <<= 1;
					if (--bits_left_in_rx == 0)
					{
						flag_rx_waiting_for_stop_bit = TRUE;
					}
				}
			}
		}
	}
}
/*END     TIMER2------------------*/

