﻿/*
 * Usart.c
 *
 * Created: 25.01.2021 11:32:26
 * Author: R0AEK
 * Ver - 1.05
 */

#include "Usart.h"

/*объявление констант и переменных---------------------------------------*/
const uint8_t abcdDefault[12] = "0804C2A6E100";	//массив ABCD code по умолчанию

static uint8_t RX_array_Data[64];				//массив принятых байтов для обмена по UART
static uint8_t TX_array_Data[64];				//массив отправляемых байтов для обмена по UART
static uint8_t data_8_EEPROM			= 0xFF;	//переменная для работы с EEPROM
uint8_t flagConnectToPC					= 0;	//флаг подключения к конфигуратору

extern const uint32_t BandsFreq[12];			//указатель на массив частот диапазонов
extern unsigned char i2c_1602;
extern uint32_t TXfreq;							//указатель  на частоту передачи для si5351
extern uint32_t freqOffset;						//указатель на частоту смещения
extern uint32_t XTAL_FREQ;						//указатель на частоту используемого кварцевого резонатора
extern uint32_t FREQ_PLLA;						//указатель на частоту ФАПЧ-А
extern uint8_t DisplayModule;					//указатель на указатель на вариант подключения 1602
extern uint8_t BandPrev;						//указатель на индекс предыдущего иапазона
extern uint8_t BandCount;						//указатель на счетчик диапазонов
extern uint8_t TimeModule;						//индекс используемого модуля часов точного времени 0-DS3231 1-GPS_NEO-6x
extern float XTAL_CORR;							//указатель на значение для корректировки частоты кварца
extern char abcdCode[12];						//массив ABCD кода диапазонов
extern uint8_t PA_Pin_Log;						//индекс логического уровня для управления внешним усилителем

extern char GPSData[100];						//строка GPS сообщения
extern uint8_t GPSDataStringReceived;			//флаг получения строки от GPS
extern uint8_t Separator;						//счетчик разделителей в GPS сообшении
extern uint8_t GPSDataIndex;					//индекс данных в строке GPS сообщения
extern uint8_t gps_Data[64];					//строка GPS данных для передачи в UART
/*END_объявление переменных----------------------------------------------*/

void USART_Init(unsigned int ubrr)
{
	/*установка скорости передачи*/
	UBRR0H = (unsigned char)(ubrr>>8);
	UBRR0L = (unsigned char)ubrr;
	/*разрешаем прерывания от RX, включение приемника и передатчика*/
	UCSR0B = (1<<RXCIE0)/*|(0<<TXCIE0)*/|(1<<RXEN0)|(1<<TXEN0);
	/*настройка и установка формата кадра:asynchronous, no parity, 1-stop bit, 8-bit data*/
	UCSR0C = (0<<UMSEL01)|(0<<UMSEL00)|(0<<UPM01)|(0<<UPM00)|(0<<USBS0)|(0<<UCSZ02)|(1<<UCSZ01)|(1<<UCSZ00);
	_delay_ms(50);
}

void handlerTXUSART(void)
{
	//проверяем буфер передачи
	if((txIn != txOut) && (UCSR0A & (1 << UDRE0)))
	{
		//в буфере что-то есть, отправляем в UART
		UDR0 = bufTX[txOut++];
		txOut &= BUFMASKTX;
	}
}

void handlerRXUSART(void)
{
	int i = 0;
	char values;
	//проверяем буфер приема
 	if(rxIn != rxOut)
 	{	
		while(rxIn != rxOut)
		{
			//читаем буфер приема пока индексы не сравняются
			values = (bufRX[rxOut++]);
			rxOut &= BUFMASKRX;
			_delay_ms(2);
			
			if (values != '\n')
			{
				//если не конец строки то собираем байты в массив...
				RX_array_Data[i++] = values;
			}
			
			if (values == '\n')
			{
				//приняли конец строки, отправляем массив на парсинг
				RX_Data_Parsing();
			}
		}
	}
}

//запись значений по умолчанию блока TX оption
void DefaultWriteTXOptions(void)
{
	eeprom_write_byte((uint8_t*)12,'R');	//позывной
	eeprom_write_byte((uint8_t*)13,'0');	//-
	eeprom_write_byte((uint8_t*)14,'A');	//-
	eeprom_write_byte((uint8_t*)15,'E');	//-
	eeprom_write_byte((uint8_t*)16,'K');	//-
	eeprom_write_byte((uint8_t*)17,' ');	//-
	eeprom_write_byte((uint8_t*)18,'N');	//QTH Локатор
	eeprom_write_byte((uint8_t*)19,'O');	//-
	eeprom_write_byte((uint8_t*)20,'1');	//-
	eeprom_write_byte((uint8_t*)21,'5');	//-
	eeprom_write_byte((uint8_t*)22,' ');	//индекс выходной мощности
	eeprom_write_byte((uint8_t*)23,'3');	//-
	eeprom_write_byte((uint8_t*)24,'0');	//индекс источника времени
	eeprom_write_byte((uint8_t*)25,'1');	//индекс задержки передачи
	eeprom_write_byte((uint8_t*)26,'0');	//управление внешним BPF
	eeprom_write_byte((uint8_t*)316,'1');	//индекс логического уровня на пине PB1
	
	for (int i=300;i<312;i++)				//запись ABCD code по умолчанию
	{
		eeprom_write_byte((uint8_t*)i,abcdDefault[i-300]);
	}
	
}

//обработка ABCD code
void ABCDDecode(void)
{
	char buffer[8] = "00000000";
	
	switch (abcdCode[BandCount-1])
	{
		case 'A':  buffer[2]='1'; buffer[3]='0'; buffer[4]='1'; buffer[5]='0';
		break;
		case 'B':  buffer[2]='1'; buffer[3]='0'; buffer[4]='1'; buffer[5]='1';
		break;
		case 'C':  buffer[2]='1'; buffer[3]='1'; buffer[4]='0'; buffer[5]='0';
		break;
		case 'D':  buffer[2]='1'; buffer[3]='1'; buffer[4]='0'; buffer[5]='1';
		break;
		case 'E':  buffer[2]='1'; buffer[3]='1'; buffer[4]='1'; buffer[5]='0';
		break;
		case 'F':  buffer[2]='1'; buffer[3]='1'; buffer[4]='1'; buffer[5]='1';
		break;
		
		default: itoa (abcdCode[BandCount-1],buffer,2);
	}
	
	if ((DisplayModule != 0)&&(flagConnectToPC == 1))
	{
		for (int i=2;i<6;i++)
			LCD_send_char(buffer[i]);
	}
	
	if (((DisplayModule == 5)||(DisplayModule == 6))&&(flagConnectToPC == 1))
	{
		//массив для строки диапазона и ABCD кода
		char str_buffer[10] = "";
		
		//метка диапазона
		switch (BandCount)
		{
			case 1:   sprintf(str_buffer, "160m  %c%c%c%c", buffer[2], buffer[3], buffer[4], buffer[5]);
			break;
			case 2:   sprintf(str_buffer, "80m   %c%c%c%c", buffer[2], buffer[3], buffer[4], buffer[5]);
			break;
			case 3:   sprintf(str_buffer, "60m   %c%c%c%c", buffer[2], buffer[3], buffer[4], buffer[5]);
			break;
			case 4:   sprintf(str_buffer, "40m   %c%c%c%c", buffer[2], buffer[3], buffer[4], buffer[5]);
			break;
			case 5:   sprintf(str_buffer, "30m   %c%c%c%c", buffer[2], buffer[3], buffer[4], buffer[5]);
			break;
			case 6:   sprintf(str_buffer, "20m   %c%c%c%c", buffer[2], buffer[3], buffer[4], buffer[5]);
			break;
			case 7:   sprintf(str_buffer, "17m   %c%c%c%c", buffer[2], buffer[3], buffer[4], buffer[5]);
			break;
			case 8:   sprintf(str_buffer, "15m   %c%c%c%c", buffer[2], buffer[3], buffer[4], buffer[5]);
			break;
			case 9:   sprintf(str_buffer, "12m   %c%c%c%c", buffer[2], buffer[3], buffer[4], buffer[5]);
			break;
			case 10:  sprintf(str_buffer, "10m   %c%c%c%c", buffer[2], buffer[3], buffer[4], buffer[5]);
			break;
			case 11:  sprintf(str_buffer, "6m    %c%c%c%c", buffer[2], buffer[3], buffer[4], buffer[5]);
			break;
			case 12:  sprintf(str_buffer, "2m    %c%c%c%c", buffer[2], buffer[3], buffer[4], buffer[5]);
			break;
			
			default: ;
		}
		
		int y = 0;
		for (int i=0;i<10;i++)
		{
			PlotChar(str_buffer[i],2,y);
			y += 12;
		}
	}
	
	//установка пинов для extBPF
	char ABCD[4];
	ABCD[0] = ABCD_A;
	ABCD[1] = ABCD_B;
	ABCD[2] = ABCD_C;
	ABCD[3] = ABCD_D;
	
	int y = 0;
	for (int i=2;i<6;i++)
	{
		switch (buffer[i])
		{
			case '0': {if (y == 3)
						PORTX_B &= ~(1 << ABCD[y]);
					  else
						PORTX_D &= ~(1 << ABCD[y]);};
			break;
			case '1': {if (y == 3)
						PORTX_B |= (1 << ABCD[y]);
					  else
						PORTX_D |= (1 << ABCD[y]);};
			break;
			
			default: ;
		}
		y++;
	}
}

//переключение диапазонов в режиме конфигуратора
void ChangeTestActiveBand(void)
{
	ABCD_Off();
	
	//выводим соответствующую информацию на LCD
	if ((DisplayModule >= 1)&&(DisplayModule <= 4))
	{
		LCD_set_pos(0,1);
		LCD_send_str("Band: ");
		
		switch (BandCount)
		{
			case 1:  LCD_send_str("160m  ");
			break;
			case 2:  LCD_send_str("80m   ");
			break;
			case 3:  LCD_send_str("60m   ");
			break;
			case 4:  LCD_send_str("40m   ");
			break;
			case 5:  LCD_send_str("30m   ");
			break;
			case 6:  LCD_send_str("20m   ");
			break;
			case 7:  LCD_send_str("17m   ");
			break;
			case 8:  LCD_send_str("15m   ");
			break;
			case 9:  LCD_send_str("12m   ");
			break;
			case 10: LCD_send_str("10m   ");
			break;
			case 11: LCD_send_str("6m    ");
			break;
			case 12: LCD_send_str("2m    ");
			break;
			
			default: ;
		}
	}
	
	ABCDDecode();
	
	//выбираем частоту текущего диапазона
	TXfreq = BandsFreq[BandCount-1];
	
	//переходим на диапазон без частоты смещения
	si5351aSetFrequency(TXfreq);
}

//запись и калькуляция регистров при корректировке частоты кварца в режиме конфигуратора
void WriteCorrXtall(void)
{
	//выбираем частоту текущего диапазона
	TXfreq = BandsFreq[BandCount-1];
	
	//переходим на частоту выбранного диапазона
	si5351aSetFrequency(TXfreq);
	
	//записываем корректировочное значение частоты используемого кварца
	data_8_EEPROM = eeprom_read_byte((uint8_t*)34);
	
	switch(data_8_EEPROM)
	{
		case '0':
					XTAL_CORR = (float)25000000 - (float)XTAL_FREQ;
					//сохраняем значение калибровки в EEPROM
					eeprom_write_float((float*)27,XTAL_CORR);
		break;
		case '1':
					XTAL_CORR = (float)27000000 - (float)XTAL_FREQ;
					//сохраняем значение калибровки в EEPROM
					eeprom_write_float((float*)27,XTAL_CORR);
		break;
		
		default: ;
	}
}

//отправка побайтно в кольцевой буфер
void SendByteToCircularBuffer(int idx)
{
	for (int i=0;i<=idx;i++)
	{
		bufTX[txIn++] = TX_array_Data[i];
		txIn &= BUFMASKTX;
	}
}

//отправка строки в USART
void SendStringDataToUSART(char *str)
{
	while(*str)
	{
		UDR0 = *str++;
		_delay_ms(1);
	}
}

//обработка данных полученных по USART
void RX_Data_Parsing(void)
{
	//если приняли идентификатор успешного подключения к конфигуратору
	{
		if ((RX_array_Data[0] == 'A')&&(RX_array_Data[1] == '1'))
		{
			if (TimeModule == 1)
			{	//если используем GPS модуль как источник времени
				/*запрещаем прием по SoftUART*/
				SUART_TurnRxOff();
			}
			
			//поднимаем флаг подключения к конфигуратору
			flagConnectToPC = 1;
			//обнуляем индекс активного диапазона при подключении к конфигуратору
			BandCount = 6;
			if ((DisplayModule >= 1)&&(DisplayModule <= 4))
			{
				//очищаем дисплей
				LCD_send_byte(0x01,0);
				LCD_set_pos(0,0);
				//выводим соответствующую информацию на LCD
				LCD_send_str("CFG-mode    ABCD");
			}
			
			if ((DisplayModule == 5)||(DisplayModule == 6))
			{
				//выводим информацию на OLED дисплей
				OLED_clear();
				PlotText(PSTR("CFG   ABCD"), 0, 0);
			}
			
			//выбираем частоту выбранного диапазона
			TXfreq = BandsFreq[BandCount-1];
			//переходим на диапазон без частоты смещения!!!
			si5351aSetFrequency(TXfreq);
			
			//переходим на выбранный диапазон
			ChangeTestActiveBand();
		}
	}
	
	//если приняли идентификатор чтения блока "Active bands" из EEPROM
	{
		if ((RX_array_Data[0] == 'A')&&(RX_array_Data[1] == '2'))
		{
			data_8_EEPROM = eeprom_read_byte((uint8_t*)4);
			if ((data_8_EEPROM != '1')&&(data_8_EEPROM != '2'))
			{	//если считанное значение не 1 или 2 то записываем значения по умолчанию, по умолчанию включен только диапазон 30m
				for (int i=0;i<=11;i++)
				{
					if (i == 4)
					{
						eeprom_write_byte((uint8_t*)i,'2');
					}
					else
					{
						eeprom_write_byte((uint8_t*)i,'1');
					}
				}
				
				//if ((DisplayModule >= 1)&&(DisplayModule <= 4))
				//{			
					////выводим соответствующую информацию на LCD
					//LCD_set_pos(0,1);
					//LCD_send_str("Bands-default!   ");
				//}
			}
			
			//значение в диапазоне 1 или 2 то отправляем в конфигуратор
			//собираем данные
			TX_array_Data[0] = 'A';
			TX_array_Data[1] = '2';
			for (int i=0;i<=11;i++)
			{
				TX_array_Data[i+2] = eeprom_read_byte((uint8_t*)i);
			}
			TX_array_Data[14] = '\n';
			
			//отправляем данные в UART
			SendByteToCircularBuffer(14); //14
			
			//if ((DisplayModule >= 1)&&(DisplayModule <= 4))
			//{
				////выводим соответствующую информацию на LCD
				//LCD_set_pos(0,1);
				//LCD_send_str("Bands-read!     ");
			//}
		}
	}
	
	//если приняли идентификатор записи блока "Active bands" в EEPROM
	{
		if ((RX_array_Data[0] == 'A')&&(RX_array_Data[1] == '3'))
		{
			for (int i=0;i<=11;i++)
			{
				eeprom_write_byte((uint8_t*)i,RX_array_Data[i+2]);
			}
			
			//if ((DisplayModule >= 1)&&(DisplayModule <= 4))
			//{
				////выводим соответствующую информацию на LCD
				//LCD_set_pos(0,1);
				//LCD_send_str("Bands-write!    ");
			//}
		}
	}
	
	//если приняли идентификатор чтения блока "TX options" из EEPROM
	{
		if ((RX_array_Data[0] == 'B')&&(RX_array_Data[1] == '1'))
		{
			//прверяем корректность записанных данных
			data_8_EEPROM = eeprom_read_byte((uint8_t*)12);
			if (data_8_EEPROM == 0xFF)
			{
				//записываем значения по умолчанию
				DefaultWriteTXOptions();
			}
			
			//собираем данные
			TX_array_Data[0] = 'B';
			TX_array_Data[1] = '1';
			for (int i=12;i<=26;i++)
			{
				TX_array_Data[i-10] = eeprom_read_byte((uint8_t*)i);
			}
			
			TX_array_Data[17] = eeprom_read_byte((uint8_t*)316);
			
			TX_array_Data[18] = '\n';
			
			//отправляем данные в UART
			SendByteToCircularBuffer(18);
			
			//if ((DisplayModule >= 1)&&(DisplayModule <= 4))
			//{
				////выводим соответствующую информацию на LCD
				//LCD_set_pos(0,1);
				//LCD_send_str("TX_opt-read!    ");
			//}
		}
	}
	
	//если приняли идентификатор записи блока "TX options" в EEPROM
	{
		if ((RX_array_Data[0] == 'B')&&(RX_array_Data[1] == '2'))
		{
			//записываем позывной, локатор, индекс выходной мощности, источник времени и флаг управления внешним BPF
			for (int i=12;i<=26;i++)
			{
				eeprom_write_byte((uint8_t*)i,RX_array_Data[i-10]);
			}
			
			//if ((DisplayModule >= 1)&&(DisplayModule <= 4))
			//{
				////выводим соответствующую информацию на LCD
				//LCD_set_pos(0,1);
				//LCD_send_str("TX_opt-write!   ");
			//}
		}
	}
	
	//если приняли идентификатор записи первого блока "WSPR data" в EEPROM
	{
		if ((RX_array_Data[0] == 'B')&&(RX_array_Data[1] == '3'))
		{
			//if ((DisplayModule >= 1)&&(DisplayModule <= 4))
			//{
				////выводим соответствующую информацию на LCD
				//LCD_set_pos(0,1);
				//LCD_send_str("WSPR ch-write...");
			//}
			
			for (int i=40;i<102;i++)
			{
				eeprom_write_byte((uint8_t*)i,RX_array_Data[i-38]);
			}
		}
	}
	
	//если приняли идентификатор записи второго блока "WSPR data" в EEPROM
	{
		if ((RX_array_Data[0] == 'B')&&(RX_array_Data[1] == '4'))
		{
			for (int i=102;i<164;i++)
			{
				eeprom_write_byte((uint8_t*)i,RX_array_Data[i-100]);
			}
		}
	}
	
	//если приняли идентификатор записи третьего блока "WSPR data" в EEPROM
	{
		if ((RX_array_Data[0] == 'B')&&(RX_array_Data[1] == '5'))
		{
			for (int i=164;i<202;i++)
			{
				eeprom_write_byte((uint8_t*)i,RX_array_Data[i-162]);
			}
			
			//if ((DisplayModule >= 1)&&(DisplayModule <= 4))
			//{
				////выводим соответствующую информацию на LCD
				//LCD_set_pos(0,1);
				//LCD_send_str("WSPR data-write! ");
			//}
		}
	}
	
	//если приняли идентификатор чтения блока "Clock generator Si5351" из EEPROM
	{
		if ((RX_array_Data[0] == 'C')&&(RX_array_Data[1] == '1'))
		{
			//прверяем корректность записанных данных Xtall
			data_8_EEPROM = eeprom_read_byte((uint8_t*)26);
			if ((data_8_EEPROM != '0')&&(data_8_EEPROM != '1'))
			{
				//записываем значение по умолчанию (25 MHz)
				eeprom_write_byte((uint8_t*)34,'0');
			}
			
			//прверяем корректность записанных данных PLLA
			data_8_EEPROM = eeprom_read_byte((uint8_t*)31);
			if ((data_8_EEPROM != '0')&&(data_8_EEPROM != '1'))
			{
				//записываем значение по умолчанию (600 MHz)
				eeprom_write_byte((uint8_t*)31,'0');
			}
			
			//прверяем корректность записанных данных FreqOffset
			data_8_EEPROM = eeprom_read_byte((uint8_t*)32);
			if ((data_8_EEPROM != '0')&&(data_8_EEPROM != '1')&&(data_8_EEPROM != '2')&&(data_8_EEPROM != '3')&&(data_8_EEPROM != '4')
				&&(data_8_EEPROM != '5')&&(data_8_EEPROM != '6')&&(data_8_EEPROM != '7')&&(data_8_EEPROM != '8')&&(data_8_EEPROM != '9'))
				//записываем значение по умолчанию (1500 Hz)
				eeprom_write_byte((uint8_t*)32,'4');
			
			//собираем данные
			TX_array_Data[0] = 'C';
			TX_array_Data[1] = '1';
			TX_array_Data[2] = eeprom_read_byte((uint8_t*)34);
			TX_array_Data[3] = eeprom_read_byte((uint8_t*)31);
			TX_array_Data[4] = eeprom_read_byte((uint8_t*)32);
			TX_array_Data[5] = '\n';
			
			//отправляем данные в UART
			SendByteToCircularBuffer(5); //5
			
			//if ((DisplayModule >= 1)&&(DisplayModule <= 4))
			//{
				////выводим соответствующую информацию на LCD
				//LCD_set_pos(0,1);
				//LCD_send_str("Si5351-read!    ");
			//}
		}
	}
	
	//если приняли идентификатор записи блока "Clock generator Si5351" в EEPROM
	{
		if ((RX_array_Data[0] == 'C')&&(RX_array_Data[1] == '2'))
		{
			//записываем Xtall, PLLA, TX freq offset
			eeprom_write_byte((uint8_t*)34,RX_array_Data[2]);
			eeprom_write_byte((uint8_t*)31,RX_array_Data[3]);
			eeprom_write_byte((uint8_t*)32,RX_array_Data[4]);
			
			//if ((DisplayModule >= 1)&&(DisplayModule <= 4))
			//{
				////выводим соответствующую информацию на LCD
				//LCD_set_pos(0,1);
				//LCD_send_str("Si5351-write!   ");
			//}
			
			//переустановка частоты с новыми параметрами
			ReadXtallPLLAfrommEEPROM();
			ChangeTestActiveBand();
		}
	}
	
	//если приняли идентификатор стирания всей EEPROM
	{
		if ((RX_array_Data[0] == 'D')&&(RX_array_Data[1] == '1'))
		{
			//if ((DisplayModule >= 1)&&(DisplayModule <= 4))
			//{
				////выводим соответствующую информацию на LCD
				//LCD_set_pos(0,1);
				//LCD_send_str("EEPROM-erase... ");
			//}
			
			//стираем EEPROM
			//включаем светик L
			PORTX_B |= (1 << LED_Status);
			for (int i=0;i<=1023;i++)
			{
				//затираем ячейку
				eeprom_write_byte((uint8_t*)i,0xFF);
			}
			//выключаем светик L
			PORTX_B &= ~(1 << LED_Status);
			
			//записываем значения по умолчанию для блока TX options
			DefaultWriteTXOptions();
			//записываем значение по умолчанию (25.000 MHz) xTal
			eeprom_write_byte((uint8_t*)34,'0');
			//записываем значение по умолчанию (600.000 MHz) PLLA freq
			eeprom_write_byte((uint8_t*)31,'0');
			//записываем значение по умолчанию (1500 Hz) TX freq offset
			eeprom_write_byte((uint8_t*)32,'5');
			
			_delay_ms(10);
			
			//собираем данные
			TX_array_Data[0] = 'D';
			TX_array_Data[1] = '1';
			TX_array_Data[2] = '\n';
			
			//отправляем данные в UART
			SendByteToCircularBuffer(2);
			
			//if ((DisplayModule >= 1)&&(DisplayModule <= 4))
			//{
				////выводим соответствующую информацию на LCD
				//LCD_set_pos(0,1);
				//LCD_send_str("EEPROM-erase-Ok!");
			//}
		}
	}
	
	//если приняли идентификатор установки времени в модуле Ds3231
	{
		if ((RX_array_Data[0] == 'E')&&(RX_array_Data[1] == '1'))
		{
			//if ((DisplayModule >= 1)&&(DisplayModule <= 4))
			//{
				////выводим соответствующую информацию на LCD
				//LCD_set_pos(0,1);
				//LCD_send_str("Set time... ");
			//}
			
			char tHour[3];
			char  tMin[3];
			char  tSec[3];
			
			//заполняем часы
			tHour[0] = (char)RX_array_Data[3];
			tHour[1] = (char)RX_array_Data[4];
			tHour[2] = '\n'; //!!!конец строки нужен для корректной работы atoi
			DateTime.Hour = atoi(tHour);
			
			//заполняем минуты
			tMin[0] = (char)RX_array_Data[6];
			tMin[1] = (char)RX_array_Data[7];
			tMin[2] = '\n'; //!!!конец строки нужен для корректной работы atoi
			DateTime.Min = atoi(tMin);
			
			//заполняем секунды
			tSec[0] = (char)RX_array_Data[9];
			tSec[1] = (char)RX_array_Data[10];
			tSec[2] = '\n'; //!!!конец строки нужен для корректной работы atoi
			
			if (DisplayModule == 0)
				DateTime.Sec = atoi(tSec) + 2; //добавляем 2 секунды для корректировки
			
			if ((DisplayModule >= 1)||(DisplayModule <= 5))
				DateTime.Sec = atoi(tSec) + 1; //добавляем 1 секунду для корректировки
			
			//добавляем задержку для корректировки
			_delay_ms(250);
			
			//записываем время, дату и параметры в Ds3231
			DS3231_Write_Time(DateTime.Hour, DateTime.Min, DateTime.Sec);
			
			//if ((DisplayModule >= 1)&&(DisplayModule <= 4))
			//{
				////выводим соответствующую информацию на LCD
				//LCD_set_pos(0,1);
				//LCD_send_str("Set time-Ok!");
			//}
		}
	}
	
	//если приняли идентификатор записи индекса подключения дисплея
	{
		if ((RX_array_Data[0] == 'F')&&(RX_array_Data[1] == '1'))
		{
			//записываем значение индекса подключения дисплея 1602
			eeprom_write_byte((uint8_t*)33,RX_array_Data[2]);
			
			if ((DisplayModule >= 1)&&(DisplayModule <= 4))
			{
				LCD_send_byte(0x01,0);	//очищаем LCD дисплей	
			}
			
			if ((DisplayModule == 5)||(DisplayModule == 6))
			{
				OLED_clear();		//очищаем OLED дисплей
			}
			
			switch (RX_array_Data[2])
			{
				case '0':  DisplayModule = 0;
				break;
				case '1':  DisplayModule = 1;
				break;
				case '2':  DisplayModule = 2;
				break;
				case '3':  DisplayModule = 3;
				break;
				case '4':  DisplayModule = 4;
				break;
				case '5':  DisplayModule = 5;
				break;
				case '6':  DisplayModule = 6;
				break;
				
				default: ;
			}
		}
	}
	
	//если приняли идентификатор переключения диапазона назад
	{
		if ((RX_array_Data[0] == 'G')&&(RX_array_Data[1] == '1'))
		{
			//запоминаем предыдущий диапазон
			BandPrev = BandCount;
			//переключаем диапазон назад
			BandCount--;
			if(BandCount < 1)
				BandCount = 12;
			
			ChangeTestActiveBand();
			
			/*//собираем данные
			TX_array_Data[0] = 'G';
			TX_array_Data[1] = '1';
			TX_array_Data[2] = (unsigned char)TestActiveBand;
			TX_array_Data[3] = '\n';
			
			//отправляем данные в UART
			SendByteToCircularBuffer(3);*/
		}
	}
	
	//если приняли идентификатор переключения диапазона вперед
	{
		if ((RX_array_Data[0] == 'G')&&(RX_array_Data[1] == '2'))
		{
			//запоминаем предыдущий диапазон
			BandPrev = BandCount;
			//переключаем диапазон назад
			BandCount++;
			if(BandCount > 12)
				BandCount = 1;
			
			ChangeTestActiveBand();
			
			/*//собираем данные
			TX_array_Data[0] = 'G';
			TX_array_Data[1] = '2';
			TX_array_Data[2] = (unsigned char)TestActiveBand;
			TX_array_Data[3] = '\n';
			
			//отправляем данные в UART
			SendByteToCircularBuffer(3);*/
		}
	}
	
	//если приняли идентификатор изменения частоты кварца вниз - 500 Гц
	{
		if ((RX_array_Data[0] == 'H')&&(RX_array_Data[1] == '1'))
		{
			XTAL_FREQ -= 500;
			
			WriteCorrXtall();
		}
		
		//если приняли идентификатор изменения частоты кварца вниз - 100 Гц
		if ((RX_array_Data[0] == 'H')&&(RX_array_Data[1] == '3'))
		{
			XTAL_FREQ -= 100;
			
			WriteCorrXtall();
		}
		
		//если приняли идентификатор изменения частоты кварца вниз - 5 Гц
		if ((RX_array_Data[0] == 'H')&&(RX_array_Data[1] == '5'))
		{
			XTAL_FREQ -= 5;
			
			WriteCorrXtall();
		}
	}
	
	//если приняли идентификатор изменения частоты кварца вверх + 500 Гц
	{
		if ((RX_array_Data[0] == 'H')&&(RX_array_Data[1] == '2'))
		{
			XTAL_FREQ += 500;
			
			WriteCorrXtall();
		}
		
		//если приняли идентификатор измеенения частоты кварца вверх + 100 Гц
		if ((RX_array_Data[0] == 'H')&&(RX_array_Data[1] == '4'))
		{
			XTAL_FREQ += 100;
			
			WriteCorrXtall();
		}
		
		//если приняли идентификатор измеенения частоты кварца вверх + 5 Гц
		if ((RX_array_Data[0] == 'H')&&(RX_array_Data[1] == '6'))
		{
			XTAL_FREQ += 5;
			
			WriteCorrXtall();
		}
	}
	
	//если приняли идентификаторы запросов GPS данных
	{
		//стоп
		{
			if ((RX_array_Data[0] == 'I')&&(RX_array_Data[1] == '0'))
			{	//отключаем передачу данных
				/*запрещаем прием по SoftUART*/
				SUART_TurnRxOff();
				
				Separator = 0;
				GPSDataIndex = 0;
				GPSDataStringReceived = 0;
								
				if ((DisplayModule >= 1)&&(DisplayModule <= 4))
				{
					LCD_set_pos(0,0);
					LCD_send_str("CFG-mode    ABCD");
				}
				
				if ((DisplayModule == 5)||(DisplayModule == 6))
				{
					//выводим информацию на OLED дисплей
					PlotText(PSTR("CFG   ABCD"), 0, 0);
				}
				
				ChangeTestActiveBand();
			}
		}
			
		//старт
		{
			if ((RX_array_Data[0] == 'I')&&(RX_array_Data[1] == '1'))
			{	//включаем передачу данных
				if ((DisplayModule >= 1)&&(DisplayModule <= 4))
				{
					LCD_set_pos(0,0);
					LCD_send_str("CFG-mode        ");
					LCD_set_pos(0,1);
					LCD_send_str("GPS to PC...    ");
				}
				
				if ((DisplayModule == 5)||(DisplayModule == 6))
				{
					//выводим информацию на OLED дисплей
					PlotText(PSTR("CFG       "), 0, 0);
					PlotText(PSTR("GPS to PC "), 2, 0);
				}
					
				//TX_array_Data[0] = 'G';
				//TX_array_Data[1] = '1';
				//TX_array_Data[2] = '\n';
					//
				////отправляем данные в UART
				//SendByteToCircularBuffer(2);
				
				Separator = 0;
				GPSDataIndex = 0;
				GPSDataStringReceived = 0;
				
				/*разрешаем прием по SoftUART*/
				SUART_TurnRxOn();
			}
		}
	}
	
	//если приняли идентификатор чтения ABCD code
	{
		if ((RX_array_Data[0] == 'J')&&(RX_array_Data[1] == '1'))
		{
			//собираем данные
			TX_array_Data[0] = 'J';
			TX_array_Data[1] = '1';
			for (int i=300;i<312;i++)
			{
				TX_array_Data[i-298] = eeprom_read_byte((uint8_t*)i);
			}
			TX_array_Data[14] = '\n';
			
			//отправляем данные в UART
			SendByteToCircularBuffer(14);
			
			//if ((DisplayModule >= 1)&&(DisplayModule <= 4))
			//{
				////выводим соответствующую информацию на LCD
				//LCD_set_pos(0,1);
				//LCD_send_str("ABCD-read!      ");
			//}
		}
	}
	
	//если приняли идентификатор записи ABCD code
	{
		if ((RX_array_Data[0] == 'J')&&(RX_array_Data[1] == '2'))
		{
			for (int i=300;i<312;i++)
			{
				eeprom_write_byte((uint8_t*)i,RX_array_Data[i-298]);
				abcdCode[i-300] = RX_array_Data[i-298];
			}
			
			//if ((DisplayModule >= 1)&&(DisplayModule <= 4))
			//{
				////выводим соответствующую информацию на LCD
				//LCD_set_pos(0,1);
				//LCD_send_str("ABCD-write!     ");
			//}
		}
	}
	
	//если приняли идентификатор на включение внешнего усилителя мощности
	{
		if ((RX_array_Data[0] == 'B')&&(RX_array_Data[1] == '6'))
		{
			//включаем внешний PA
			if (PA_Pin_Log == '1')
				PORTX_B |= (1 << PA_Pin);
			if (PA_Pin_Log == '0')
				PORTX_B &= ~(1 << PA_Pin);
			
			if ((DisplayModule >= 1)&&(DisplayModule <= 4))
			{
				//выводим соответствующую информацию на LCD
				LCD_set_pos(0,0);
				LCD_send_str("CFG-mode PA ABCD");
			}
			
			if ((DisplayModule == 5)||(DisplayModule == 6))
			{
				//выводим информацию на OLED дисплей
				PlotText(PSTR("CFG P ABCD"), 0, 0);
			}
		}
	}
	
	//если приняли идентификатор на отключение внешнего усилителя мощности
	{
		if ((RX_array_Data[0] == 'B')&&(RX_array_Data[1] == '7'))
		{
			//выключаем внешний PA
			if (PA_Pin_Log == '1')
				PORTX_B &= ~(1 << PA_Pin);
			if (PA_Pin_Log == '0')
				PORTX_B |= (1 << PA_Pin);
			
			if ((DisplayModule >= 1)&&(DisplayModule <= 4))
			{
				//выводим соответствующую информацию на LCD
				LCD_set_pos(0,0);
				LCD_send_str("CFG-mode    ABCD");
			}
			
			if ((DisplayModule == 5)||(DisplayModule == 6))
			{
				//выводим информацию на OLED дисплей
				PlotText(PSTR("CFG   ABCD"), 0, 0);
			}
		}
	}
	
	//если приняли идентификатор на запись логического уровня на пине PB1 при передаче
	{
		if ((RX_array_Data[0] == 'B')&&(RX_array_Data[1] == '8'))
		{
			eeprom_write_byte((uint8_t*)316,RX_array_Data[2]);
			PA_Pin_Log = RX_array_Data[2];
			
			//выключаем внешний PA
			{
				if (PA_Pin_Log == '1')
					PORTX_B &= ~(1 << PA_Pin);
				if (PA_Pin_Log == '0')
					PORTX_B |= (1 << PA_Pin);
			}
			
			//if ((DisplayModule >= 1)&&(DisplayModule <= 4))
			//{
				////выводим соответствующую информацию на LCD
				//LCD_set_pos(0,1);
				//LCD_send_str("Pin PB1 log set!");
			//}
		}
	}
}

//отправка GPS данных на PC
void GPStoPC(void)
{
	//если строка GPS сообщения получена корректно, то передаем данные
	if (GPSDataStringReceived == 1)
	{
		Parsing_GPS_Message();
		
		for (int i=0;i<62;i++)
		TX_array_Data[i] = gps_Data[i];
		
		TX_array_Data[62] = '\n';
		//отправляем данные в UART
		SendByteToCircularBuffer(62);
	}
	
	/*разрешаем прием по SoftUART*/
	SUART_TurnRxOn();
}

//мониторинг в UART
// void UART_Monitor(char *str)
// {
// 	for (int i=0;i<15;i++)
// 		TX_array_Data[i] = str[i];
// 	
// 	TX_array_Data[15] = '\n';
// 	
// 	SendByteToCircularBuffer(15);
// }