/*
 * i2c.c
 *
 * Created: 25.01.2021 11:32:26
 * Author: R0AEK
 * Ver - 1.05
 */ 

#include "Lcd_1602.h"

extern uint8_t TransmitDelay;
extern uint8_t PrevTimeStartTX;
extern uint8_t FirstTX;
extern uint8_t DisplayModule;
extern uint8_t flagConnectToPC;

void i2cInit()
{
	/*
	     i2c   400 
	    TWBR = ((F_CPU/F_SCL)-16)/2 = ((16000/100)-16)/2 = 72  0x48
	    1602  <= 65
	    si5351 (QRPLab) = 92
	*/
	TWBR = 65; //65
	TWSR = 0;
	TWDR = 0xFF;
	PRR  = 0;
}

uint8_t i2cStart()
{
	TWCR = (1<<TWINT) | (1<<TWSTA) | (1<<TWEN);
	while (!(TWCR & (1<<TWINT))) ;
	return (TWSR & 0xF8);
}

void i2cStop()
{
	TWCR = (1<<TWINT) | (1<<TWEN) | (1<<TWSTO);
	while ((TWCR & (1<<TWSTO))) ;
}

uint8_t i2cByteSend(uint8_t data)
{
	TWDR = data;
	TWCR = (1<<TWINT) | (1<<TWEN);
	while (!(TWCR & (1<<TWINT))) ;
	return (TWSR & 0xF8);
}

uint8_t i2cByteRead()
{
	TWCR = (1<<TWINT) | (1<<TWEN);
	while (!(TWCR & (1<<TWINT))) ;
	return (TWDR);
}

uint8_t i2cSendRegister(uint8_t reg, uint8_t data)
{
	uint8_t stts;
	stts = i2cStart();
	if (stts != I2C_START) return 1;
	stts = i2cByteSend(I2C_WRITE);
	if (stts != I2C_SLA_W_ACK) return 2;
	stts = i2cByteSend(reg);
	if (stts != I2C_DATA_ACK) return 3;
	stts = i2cByteSend(data);
	if (stts != I2C_DATA_ACK) return 4;
	i2cStop();
	return 0;
}

uint8_t i2cCurrentByteRead()
{
	TWCR = (1<<TWINT)|(1<<TWEN)|(1<<TWEA);
	while (!(TWCR & (1<<TWINT)));			//   TWIN
	return TWDR;							//  
}

/*    i2c    */
void I2C_SendByteByADDR(unsigned char c,unsigned char addr)
{
	i2cStart();			//  START
	i2cByteSend(addr);	//     +  -
	i2cByteSend(c);		//  
	i2cStop();			//  STOP
}

/*    DS-3231  i2c*/
unsigned char RTC_ConvertFromBinDec(unsigned char c)
{
	unsigned char ch = ((c/10)<<4)|(c%10);
	return ch;
}
 
unsigned char RTC_ConvertFromDec(unsigned char c)
{
	unsigned char ch = ((c>>4)*10+(0b00001111&c));
	return ch;
}

void DS3231_Write_Time(uint8_t rtcH, uint8_t rtcM, uint8_t rtcS)
{
	i2cStart();									//
	i2cByteSend(0xD0);							// 
	i2cByteSend(0x00);							//  0x00
	i2cByteSend(RTC_ConvertFromBinDec(rtcS));	//
	i2cByteSend(RTC_ConvertFromBinDec(rtcM));	//
	i2cByteSend(RTC_ConvertFromBinDec(rtcH));	//
	i2cStop();									//
}

uint8_t DS3231_Read_Time()
{
	char ds3231_buffer[16] = "";				//      DS-3231
	
	I2C_SendByteByADDR(0x00,0xD0);	
	
	i2cStart();									//
	i2cByteSend(0xD1);							// 
	rtcSec  = i2cCurrentByteRead();				//
	rtcMin  = i2cCurrentByteRead();				//
	rtcHour = i2cByteRead();					// -  
	i2cStop();									//
	
	rtcHour = RTC_ConvertFromDec(rtcHour);
	rtcMin  = RTC_ConvertFromDec(rtcMin);
	rtcSec  = RTC_ConvertFromDec(rtcSec);
	
	
	
	//  LCD 
	if ((DisplayModule >= 1)&&(DisplayModule <= 4))
	{
		//      LCD 
		sprintf(ds3231_buffer, "Waiting %02i:%02i:%02i", rtcHour, rtcMin, rtcSec);
		
		LCD_set_pos(0,0);
		LCD_send_str(ds3231_buffer);
	}
	
	//  OLED 
	if ((DisplayModule == 5)||(DisplayModule == 6))
	{
		//      OLED 
		sprintf(ds3231_buffer, " %02i:%02i:%02i \n", rtcHour, rtcMin, rtcSec);
		
		//  
		if ((rtcSec == 0)||(rtcSec == 10)||(rtcSec == 20)||(rtcSec == 30)||
			(rtcSec == 40)||(rtcSec == 50))
			PlotText(PSTR("  Waiting  "), 0, 0);
			
		if ((rtcSec == 5)||(rtcSec == 15)||(rtcSec == 25)||(rtcSec == 35)||
			(rtcSec == 35)||(rtcSec == 45)||(rtcSec == 55))
			PlotText(PSTR("  DS-3231  "), 0, 0);
			
		int y = 0;
		for (int i=0;i<10;i++)
		{
			PlotChar(ds3231_buffer[i],2,y);
			y += 13;
		}
	}
	
	//     (1  2   )
	if((rtcMin % 2 == 0)&&(rtcSec == 1))
	{
		if ((TransmitDelay != '0')&&(FirstTX == 0))
		{	//         
			PrevTimeStartTX++;
		}
		return 1;	//   
	}
	else
		return 0;	//    
}
  

