/*
Author: Hans Summers, 2015
Website: http://www.hanssummers.com

A very very simple Si5351a demonstration
using the Si5351a module kit http://www.hanssummers.com/synth
Please also refer to SiLabs AN619 which describes all the registers to use

Mod: R0AEK, 2021
Ver - 1.05
*/

#include <inttypes.h>
#include "Main.h"
#include "i2c.h"
#include "si5351a.h"

/*   ---------------------------------------*/
uint32_t XTAL_FREQ	= 0;		//    (  25.000 )
uint32_t FREQ_PLLA	= 0;		// - (600/900 ) ( 900 ,   , "" !!!)

uint32_t pllFreq;
uint32_t xtalFreq;
uint32_t l;
float f;
uint8_t mult;
uint32_t num;
uint32_t denom;
uint32_t divider;

extern uint8_t BandPrev;		//    
extern uint8_t BandCount;		//    
extern uint8_t flagConnectToPC;	//     
extern uint8_t Uses6mBand;		//     6m
extern uint8_t Uses2mBand;		//     2m
extern uint32_t StartFreq;		//     si5351 (1 )
/*END    -----------------------------------*/

/*
    PLL  mult, num  denom
mult   15..90
num   0..1,048,575 (0xFFFFF)
denom   0..1,048,575 (0xFFFFF)
*/
void setupPLL(uint8_t pll, uint8_t mult, uint32_t num, uint32_t denom)
{
	uint32_t P1;					// PLL   P1
	uint32_t P2;					// PLL   P2
	uint32_t P3;					// PLL   P3

	P1 = (uint32_t)(128 * ((float)num / (float)denom));
	P1 = (uint32_t)(128 * (uint32_t)(mult) + P1 - 512);
	P2 = (uint32_t)(128 * ((float)num / (float)denom));
	P2 = (uint32_t)(128 * num - denom * P2);
	P3 = denom;

	i2cSendRegister(pll + 0, (P3 & 0x0000FF00) >> 8);
	i2cSendRegister(pll + 1, (P3 & 0x000000FF));
	i2cSendRegister(pll + 2, (P1 & 0x00030000) >> 16);
	i2cSendRegister(pll + 3, (P1 & 0x0000FF00) >> 8);
	i2cSendRegister(pll + 4, (P1 & 0x000000FF));
	i2cSendRegister(pll + 5, ((P3 & 0x000F0000) >> 12) | ((P2 & 0x000F0000) >> 16));
	i2cSendRegister(pll + 6, (P2 & 0x0000FF00) >> 8);
	i2cSendRegister(pll + 7, (P2 & 0x000000FF));
}

/*
 MultiSynth     R- 
R- -   ,     , . #define  si5351a.h
*/
void setupMultisynth(uint8_t synth, uint32_t divider, uint8_t rDiv)
{
	uint32_t P1;
	uint32_t P2;
	uint32_t P3;

	P1 = 128 * divider - 512;
	P2 = 0;							// P2 = 0, P3 = 1     
	P3 = 1;

	i2cSendRegister(synth + 0,   (P3 & 0x0000FF00) >> 8);
	i2cSendRegister(synth + 1,   (P3 & 0x000000FF));
	i2cSendRegister(synth + 2,   ((P1 & 0x00030000) >> 16) | rDiv);
	i2cSendRegister(synth + 3,   (P1 & 0x0000FF00) >> 8);
	i2cSendRegister(synth + 4,   (P1 & 0x000000FF));
	i2cSendRegister(synth + 5,   ((P3 & 0x000F0000) >> 12) | ((P2 & 0x000F0000) >> 16));
	i2cSendRegister(synth + 6,   (P2 & 0x0000FF00) >> 8);
	i2cSendRegister(synth + 7,   (P2 & 0x000000FF));
}

/*
 AN619    - 0x80    
 16/17/18  7    CLK0/1/2 
*/
// void si5351aOutputOff(uint8_t clk)
// {
// 	i2cSendRegister(clk, 0x80);
// }

void si5351FirstCalc(uint32_t frequency)
{
	xtalFreq = XTAL_FREQ;
	
	/*  . 900,000,000 Hz -    PLLA*/
	divider = FREQ_PLLA / frequency;
	
	/* */
	if (divider % 2) divider--;

	/*  pllFreq:  *   */
	pllFreq = divider * frequency;

	/*     pllFreq
	   :
	1: mult -  ,      15..90
	2: num  denom -  ,   
					    20 bits ( 0..1048575)
	3:   mult + num / denom
	          1048575*/
	mult = pllFreq / xtalFreq;
	l = pllFreq % xtalFreq;
	f = l;
	f *= 1048575;
	f /= xtalFreq;
	num = f;
	denom = 1048575;
}

/*
,  CLK0      
      1MHz  150MHz
: si5351aSetFrequency(10000000);
   CLK0    10MHz

  PLLA
 MultiSynth 0
    CLK0/1/2     
*/
void si5351aSetFrequency(uint32_t frequency)
{
// 	uint32_t pllFreq;
// 	uint32_t xtalFreq = XTAL_FREQ;
// 	uint32_t l;
// 	float f;
// 	uint8_t mult;
// 	uint32_t num;
// 	uint32_t denom;
// 	uint32_t divider;

// 	/*  . 900,000,000 Hz -    PLLA*/
// 	divider = FREQ_PLLA / frequency;
// 	
// 	/* */
// 	if (divider % 2) divider--;
// 
// 	/*  pllFreq:  *   */
// 	pllFreq = divider * frequency;
// 
// 	/*     pllFreq
// 	   :
// 	1: mult -  ,      15..90
// 	2: num  denom -  ,   
// 					    20 bits ( 0..1048575)
// 	3:   mult + num / denom
// 	          1048575*/
// 	mult = pllFreq / xtalFreq;
// 	l = pllFreq % xtalFreq;
// 	f = l;
// 	f *= 1048575;
// 	f /= xtalFreq;
// 	num = f;
// 	denom = 1048575;

	si5351FirstCalc(frequency);

	/* PLLA    */
	setupPLL(SI_SYNTH_PLL_A, mult, num, denom);
	
	/*  MultiSynth 0   
	   R     2,   1..128.
	  SI_R_DIV1....SI_R_DIV128 (.   si5351a.h)
	      1 ,    SI_R_DIV128*/
	
	/*       */
	{
		if (frequency == StartFreq)
		{
			/* CLK0*/
			setupMultisynth(SI_SYNTH_MS_0, divider, SI_R_DIV_1);
			/*  CLK0 ( 0x4F)
			    MultiSynth 0   PLLA*/
			i2cSendRegister(SI_CLK0_CONTROL, 0x4F | SI_CLK_SRC_PLL_A);
			
			if (Uses6mBand == '1')
			{
				/* CLK1*/
				setupMultisynth(SI_SYNTH_MS_1, divider, SI_R_DIV_1);
				/*  CLK1 ( 0x4F)
				    MultiSynth 0   PLLA*/
				i2cSendRegister(SI_CLK1_CONTROL, 0x4F | SI_CLK_SRC_PLL_A);
			}
			
			if (Uses2mBand == '1')
			{
				/*!!!    2m,     CLK2    */
				//if ((Uses2mBand == '1')&&(flagConnectToPC != 1))
				//{
				//si5351FirstCalc(144489000);
				/* PLLA    */
				//setupPLL(SI_SYNTH_PLL_A, mult, num, denom);
				//}
				
				/* CLK2*/
				setupMultisynth(SI_SYNTH_MS_2, divider, SI_R_DIV_1);
				/*  CLK2 ( 0x4F)
				    MultiSynth 0   PLLA*/
				i2cSendRegister(SI_CLK2_CONTROL, 0x4F | SI_CLK_SRC_PLL_A);
			}
		}
	}
	
	/*     ,        */
	if (flagConnectToPC == 1)
	{
		i2cSendRegister(SI_CLK0_CONTROL, 0x80);
		i2cSendRegister(SI_CLK1_CONTROL, 0x80);
		i2cSendRegister(SI_CLK2_CONTROL, 0x80);
	}
	
	/*       */
	/*HF*/
	{
		if ((BandCount >= 1)&&(BandCount <= 10))
		{
			/*    CLK0*/
			setupMultisynth(SI_SYNTH_MS_0, divider, SI_R_DIV_1);
			
			/* PLL -    ,    
			   PLL,    */
			/*        ,  PLL*/
			if (flagConnectToPC == 1)
			{
				if (BandPrev != BandCount)
				{
					i2cSendRegister(SI_PLL_RESET, 0xA0);
				}
			}
			
			/*  CLK0 (0x4F)
			    MultiSynth 0   PLLA*/
			i2cSendRegister(SI_CLK0_CONTROL, 0x4F | SI_CLK_SRC_PLL_A);
		}
	}
	
	/*6m*/
	{
		if (BandCount == 11)
		{
			/* 6m   CLK1*/
			setupMultisynth(SI_SYNTH_MS_1, divider, SI_R_DIV_1);
			
			/* PLL -    ,    
			   PLL     */
			/*        ,  PLL*/
			if (flagConnectToPC == 1)
			{
				if (BandPrev != BandCount)
				{
					i2cSendRegister(SI_PLL_RESET, 0xA0);
				}
			}
			
			/*  CLK1 (0x4F)
			    MultiSynth 0   PLLA*/
			i2cSendRegister(SI_CLK1_CONTROL, 0x4F | SI_CLK_SRC_PLL_A);
		}
	}
	
	/*2m*/
	{
		if (BandCount == 12)
		{
			/* 2m   CLK2*/
			setupMultisynth(SI_SYNTH_MS_2, divider, SI_R_DIV_1);
			
			/* PLL -    ,    
			   PLL     */
			/*        ,  PLL*/
			if (flagConnectToPC == 1)
			{
				if (BandPrev != BandCount)
				{
					i2cSendRegister(SI_PLL_RESET, 0xA0);
				}
			}
			
			/*  CLK2 (0x4F)
			    MultiSynth 0   PLLA*/
			i2cSendRegister(SI_CLK2_CONTROL, 0x4F | SI_CLK_SRC_PLL_A);
		}
	}
}